"Python3 program to split a metafile into 2 parts (train/test), at random."

##  RANDMETS :
##
##  Python3 metafile splitting program :
##  Copyright 2012, R.S. Forsyth.
##  Released under GNU Public licence, version 3:
##  http://www.gnu.org/licenses/gpl.txt
##  first version : 06/07/2008
##  last revision : 22/04/2017


ShowGNU = 0  ##  change to 1 2c licence statement
GNUblurb = """
  This copyrighted program is free software: you can redistribute it &/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.txt>.
"""
if ShowGNU:
    print(GNUblurb)
    
import  math, os, string, sys
import  time, random

##  local libraries :
from globdefs import *
from glib import numeric, numerize
from glib import openfile, paraget, parashow, showobj
from glib import sibpath, toklines


##  parameter info :
Parapath = os.path.expanduser('~')+os.sep+"parapath"
##  will be added to parameter file search list.
##  List of parameters that need 2b set, with defaults & lo,hi limits if appropriate :
Paralist = [
    ("atomize",1,0,1),
    ("metafile",''),  ##  info about main corpus
    ("filextn",''),
    ("jobname","randmets"),  ##  name to link related files
    ("randfrac",1.0/Gold,0,1),  ##  fraction to send to first o/p metafile
    ("outmeta1",''),  ##  first o/p meta-file
    ("outmeta2",''),  ##  second o/p metafile
    ("outpath",os.pardir+os.sep+"op"),  ##  default o/p drive/path
    ("outfile",os.pardir+os.sep+"op"+os.sep+"randmets.txt"),  ##  listing file
    ##  logfile ?
    ("zonk",0,0,1)  ##  just a sentinel
    ]
##  Each item above is a tuple: name,default,minvalue,maxvalue.
##  Contents of parafile will take precedence over these defaults,
##  but parameters not in Paralist won't be used even if in file.
##  0/1 used as True/False (lazy but easy!).
##  parafile syntax, line by line:
##  paraname <blank(s)> paraval


##  Global vars :
Version = "1.2"
Mindocs = 2  ##  minimum number of docs in each corpus
Outtail1 = "_met1"
Outtail2 = "_met2"
Feffmarx = ["\ufeff","\uffef"]  ##  to detect Windows faffing around with feff
Normcols = ["prepath","filename","doctype"]  ##  expected column names
##  + creator, workname, partcode, language, xval, yval, horz, vert ??
Feff = "\ufeff"  ##  nasty byte-mark left by Windows (??)


##  Main program :
##
def main():
    "Main line."
    
    ##  First some odds & ends concerned with orientation :
    args = sys.argv[:]          ##  gets command-line arguments
    na = len(args)
    progname = args[0]
    dateline = time.asctime(time.localtime(time.time()))
    print(progname,Version,dateline)
    print("command-line args. =",na)
    prepath = os.path.dirname(progname)  ##  directory from which program runs
    print("prepath :",prepath)
    ##  print(sys.path)
    whereat = os.getcwd()  ##  working directory
    print("working folder: ",whereat)
    sidepath = sibpath(whereat,"parapath")
    
    if na < 2:
        print("script usage:  python " + progname + " <parafile>")
        args.append(input("please give parameter file name : "))
    na = len(args)
    paraname = args[1]
    me = sys.argv[0]
    prepaths = [sidepath]
    if not whereat in prepaths:
        prepaths.append(whereat)
    prepaths = prepaths + [os.pardir,os.curdir]
    if not Parapath in prepaths:
        prepaths.append(Parapath)
    prepaths = prepaths + [os.path.expanduser('~')]
    print("Paths to search for parameter file :")
    print(prepaths)
    
    startime = time.time()
    
    parafile = openfile(paraname,prepaths,[".pf",".par",".txt",'.'],'r')
    ##  print(parafile)
    if parafile == None:
        print("Cannot proceed without a parameter file!",NL,"Program halts.")
        return  -1

    ##  Read parameter settings :
    paradat = paraget(Paralist,parafile)
    ##  print(list(paradat.__dict__.items()),NL)
    parafile.close()
    
    ##  do some tidying of paradat :
    paradat.dateline = dateline
    paradat.progname = progname
    if hasattr(paradat,"casefold"):
        paradat.foldcase = paradat.casefold
    elif hasattr(paradat,"foldcase"):
        paradat.casefold = paradat.foldcase
    if hasattr(paradat,"filextn"):
        if len(paradat.filextn) > 0 and not paradat.filextn.startswith('.'):
            paradat.filextn = '.' + paradat.filextn
            ##  ensures initial fullstop
    else:
        paradat.filextn = ''  ##  must have filextn, even if empty
        ##  empty string means read any file type
    if not paradat.outpath.endswith(os.path.sep):
        paradat.outpath = paradat.outpath + os.path.sep
    ##  outpath redundant ??

    ##  open listing file :
    paradat.outname = paradat.outfile[:]
    paradat.outfile = open(paradat.outfile,'w',encoding="utf8")

    ##  prepare o/p files :
    if paradat.outmeta1:
        paradat.outname1 = paradat.outmeta1
    else:  ##  make a default name
        paradat.outname1 = paradat.outpath + paradat.jobname + Outtail1 + ".dat"
    paradat.out1 = open(paradat.outname1,'w',encoding="utf-8")
    if paradat.outmeta2:
        paradat.outname2 = paradat.outmeta2
    else:  ##  make a default name
        paradat.outname2 = paradat.outpath + paradat.jobname + Outtail2 + ".dat"
    paradat.out2 = open(paradat.outname2,'w',encoding="utf-8")
    
    ##  read meta-data file :
    metaname = getattr(paradat,"metafile",'')
    if not metaname:
        print("Can't proceed without metafile!")
        print("Program halts.")
        return  -2
    metafile = open(metaname,'r',encoding="utf8")
    metlines = metafile.readlines()  ##  just treat as text file
    metafile.close()

    parashow(paradat,paradat.outfile)

    ##  weed out blank lines :
    goodmets = []
    for ml in metlines:
        metline = ml.strip()
        if len(metline) > 0:
            goodmets.append(metline)
    gm = len(goodmets)
    indices = list(range(1,gm))  ##  topline is header
    n1 = round(gm*paradat.randfrac)
    indices1 = random.sample(indices,n1)
    print(n1)

    ##  distribute lines at random :
    m1 = m2 = 0
    if gm > 0:  ##  headers
        paradat.out1.write(goodmets[0]+NL)
        paradat.out2.write(goodmets[0]+NL)
    for mc,metline in enumerate(goodmets):
        lm = len(metline)  ##  already stripped
        if lm < 1:  ##  blank line
            continue  ##  error ?!
        if mc in indices1:
            paradat.out1.write(metline + NL)
            m1 += 1
        elif mc > 0:
            paradat.out2.write(metline + NL)
            m2 += 1   

    paradat.out1.close()
    paradat.out2.close()
    mc = gm - 1  ##  headline doesn't count
    
    ##  finale :
    print(mc,"entries read.")
    print(m1,"entries sent to :",paradat.out1.name)
    print(m2,"entries sent to :",paradat.out2.name)
    print(progname,"done on",time.asctime(time.localtime(time.time())))
    print("after",time.time()-startime,"seconds.")  ##  secs since kickoff
    
    return  0




##  Call main and attempt to honour exit status:
if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        sys.exit(1)
